var adobe = adobe ? adobe : {dps:{}};
if (!adobe.dps) {
	adobe['dps'] = {};
}

adobe.dps.Store = function()
{
	this.callbackDictionary = new Object();
};


adobe.dps.Store.prototype.FolioState = 
{
	INVALID				: 0,
    UNAVAILABLE			: 50,    /** The folio is not yet available for purchase */
    PURCHASABLE			: 100,   /** Can be purchased and downloaded */
    PURCHASING			: 101,   /** There is an active or paused Purchase Transaction for the folio */
    DOWNLOADABLE		: 200,   /** The folio is free, or its Purchase Transaction completed successfully */
    DOWNLOADING			: 201,   /** There is an active or paused Download Transaction for the folio */
    EXTRACTABLE			: 400,   /** The folio content is stored locally. */
    EXTRACTING			: 401,   /** There is an active or pause Extraction Transaction for the folio */
    VIEWABLE			: 1000,  /** The folio is can be loaded in the viewer */
    UPDATEDOWNLOADABLE	: 1500,  /** The folio is viewable but can be updated */
    UPDATEDOWNLOADING	: 1501,  /** There is an active update download for this folio */
    UPDATEEXTRACTABLE	: 1502,  /** The folio is viewable but has an update that can be installed */
    UPDATEEXTRACTING	: 1503   /** There is an active update extraction for this folio */
};

/**
* Get library state information for the folios in the library.
*
* @param callback:	Function to receive the list of folios.
*		 each folio has these properties: id, productId, price, state, title, description, broker, thirdPartyEntitled, coverData, filter, canArchive
*		 id is the folio unique id
*        productId is the store product identifier
*		 price is a localized string representing the price of the folio
*        state has the following values:
*		     100, purchasable
*		     >= 200 < 1000 entitled, downloadable, installable.
*		     >= 1000 viewable, update available.
*        title is the title title extracted from the metadata
*        description is the description extracted from the metadata
*        broker is the authority that will entitle the folio (i.e. appleStore)
*        thirdPartyEntitled indicates whether this folio is entitled by the entitlement server, where applicable
*		 coverDate is the coverDate extracted from the metadata if one was provided
*        filter is the filter value extracted from the metadata
*        canArchive is a boolean that describes if a folio can have archive called on it.
*            - This will be true for folios that have been previewed even through the state will be 100 (Purchasable)
*		 NOTE: If no current or cached information is available, the function
*		 will return an empty list
*/
adobe.dps.Store.prototype.getFolioData = function(callback)
{
	var token = this.getResponseToken("getFolioData");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getFolioData", token:token}});
};

/**
 *  Get library state information for the filters in the library
 *  @param callback: Function to receive the list of filter
 *      each filter is a string
 *      NOTE: If no current or cached information is available, the function will
 *      return an empty list
 */
adobe.dps.Store.prototype.getFilterData = function(callback)
{
	var token = this.getResponseToken("getFilterData");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getFilterData", token:token}});
};

/**
* Get library state information for the subscriptions available in the library.
*
* @param callback:	Function to receive the list of subscriptions.
*		 each subscription has these properties: productId, title, duration, price, owned, active
*		 where productId is the store product identifier
*		 title is the formatted title of the subscription
*		 duration is the formatted duration of the subscription (eg. 3 Months, 1 Week etc)
*		 price is a localized string representing the price of the folio
*		 owned is a boolean signifying that this subscription has been owned by the user
*		 active is a boolean signifying that this subscription is currently active
*		 NOTE: If no current or cached information is available, the function
*		 will return an empty list
*/
adobe.dps.Store.prototype.getSubscriptionData = function(callback)
{
	var token = this.getResponseToken("getSubscriptionData");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getSubscriptionData", token:token}});
};

/**
* Download or view the folio with the given product ID. If the folio is already on the device
* the viewer will open it for viewing otherwise the viewer will start the download of the folio.
* In either case the store web view will get closed immediately.
* Notice that there is no callback for this function. If the view/download fails the error
* is handled by the viewer in whatever way is appropriate.
* @param: folioProductId A string containing the product id of the folio to view or download.
*/
adobe.dps.Store.prototype.viewFolio = function(folioProductId)
{
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"viewFolio", folioProductId:folioProductId}});
};

/**
 * Check if a folio with the given product ID supports being downloaded as a free preview.
 * @param: folioProductId A string containing the product id of the interested folio.
 * @param: callback Function that will be called with this signature:
 *		  {"result":resultVal} where resultVal is true or false
 */
adobe.dps.Store.prototype.supportsContentPreview = function(folioProductId, callback)
{
	var token = this.getResponseToken("supportsContentPreview");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"supportsContentPreview", folioProductId:folioProductId, token:token}});
};

/**
* Download or view the free version of an un-entitled folio with the given product ID. 
* If the folio is already on the device the viewer will open it for viewing otherwise 
* the viewer will start the download of the folio. In either case the store web view will get closed immediately.
* @param: folioProductId A string containing the product id of the folio to view or download.
* @param: callback Function that will be called if the preview cannot be started:
*		  {"result":failed}
*/
adobe.dps.Store.prototype.previewContent = function(folioProductId, callback)
{
    var token = this.getResponseToken("previewContent");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"previewContent", folioProductId:folioProductId, token:token}});
};

/**
* Purchase a folio with the given product ID.
* @param: folioProductId A string containing the product id of the folio to buy.
* @param: callback Function that will be called with this signature:
*		  {"result":status} where status is one of "succeeded", "failed", "cancelled", "refunded"
* @param: withAnalytics An optional boolean indicating whether you want the Viewer app to track analytics for this event.
*         If you track analytics via JavaScript in your own store, pass in false.
*         Default value: true
*/
adobe.dps.Store.prototype.buyFolio = function(folioProductId, callback, withAnalytics)
{
	withAnalytics = (typeof(withAnalytics) != 'undefined') ? withAnalytics : true;
	
	var token = this.getResponseToken("buyFolio");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"buyFolio", folioProductId:folioProductId, withAnalytics:withAnalytics, token:token}});
};

/**
* Archive a folio with the given product ID.
* @param: folioProductId A string containing the product id of the folio to archive.
* @param: callback Function that will be called with this signature:
*		  {"result":status} where status is one of "succeeded", "failed"
*/
adobe.dps.Store.prototype.archiveFolio = function(folioProductId, callback)
{
	var token = this.getResponseToken("archiveFolio");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"archiveFolio", folioProductId:folioProductId, token:token}});
};

/**
* Purchase a subscription with the given product ID.
* @param: subscriptionProductId A string containing the product id of the subscription to buy.
* @param: callback Function that will be called with this signature:
*		  {"result":status} where status is one of "succeeded", "failed", "cancelled", "refunded", "notsupported"
* @param: withAnalytics An optional boolean indicating whether you want the Viewer app to track analytics for this event.
*         If you track analytics via JavaScript in your own store, pass in false.
*         Default value: true
*/
adobe.dps.Store.prototype.buySubscription = function(subscriptionProductId, callback, withAnalytics)
{
	withAnalytics = (typeof(withAnalytics) != 'undefined') ? withAnalytics : true;
	
	var token = this.getResponseToken("buySubscription");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"buySubscription", subscriptionProductId:subscriptionProductId, withAnalytics:withAnalytics, token:token}});
};

/**
* Get auth token and user name. Either value may be empty.
* @param callback: Function that will be called with the user info:
		  {"authToken":currentAuthToken, "userName": currentUsername}
*/
adobe.dps.Store.prototype.getUserInfo = function(callback)
{
	var token = this.getResponseToken("getUserInfo");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getUserInfo", token:token}});
};

/**
* Set the auth token and user name and let the viewer persist them.
* Notice that there is no callback for this function invalid values for either
* authToken or userName will make this function fail silently.
* @param authToken: String containing the auth token or undefined in order to sign out.
* @param userName:	String containing the user name used to sign in or undefined to clear the persisted user name.
*/
adobe.dps.Store.prototype.setUserInfo = function(authToken, userName)
{
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"setUserInfo", authToken:authToken, userName:userName }});
};

/**
* Get a unique identifier string for the device the viewer is running on.
 * @param: callback Function that will be called with this signature:
 *		  deviceId where deviceId is the string containing the unique device ID
*/
adobe.dps.Store.prototype.getDeviceId = function(callback)
{
	var token = this.getResponseToken("getDeviceId");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getDeviceId", token:token}});
};

/**
* Initiate a refresh of entitlements in the viewer.
* Notice that there is no callback for this function.
* On error, will fail silently
*/
adobe.dps.Store.prototype.refreshEntitlements = function()
{
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"refreshEntitlements"}});
};

/**
* Initiate library update.
* The response will not be sent through a callback function but through the 
* libraryUpdateComplete since a library update can be initiated by different actions
* and the interface should be notified when this happens.
*/

adobe.dps.Store.prototype.updateLibrary = function(handler)
{
	if (handler) {
		this.registerLibraryUpdateCompleteHandler(handler);
	}
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"updateLibrary"}});
};

adobe.dps.Store.prototype.goToLibrary = function()
{
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"goToLibrary"}});
};

/** Register the handler that will notify that a libraryUpdate has occurred. */
adobe.dps.Store.prototype.registerLibraryUpdateCompleteHandler = function(handler)
{
	// We do not use token here in order to have the same callback used for all updateLibraryCalls;
	this.callbackDictionary["updateLibraryComplete"] = handler;
};

adobe.dps.Store.prototype.unregisterLibraryUpdateCompleteHandler = function(){
	delete this.callbackDictionary["updateLibraryComplete"];
};

adobe.dps.Store.prototype.updateLibraryComplete = function(data){
	if( this.callbackDictionary["updateLibraryComplete"] != null ) {
		this.callbackDictionary["updateLibraryComplete"](data);
	}
};

/**
 * Request the path for a local preview image. the image will be downloaded if it is not already cached.
 * @param productId A string containing the product id of the subscription to buy.
 * @param portrait An boolean indicating whether you want a portrait or landscape preview image
 * @param width The request width of the preview (Note: This property may be ignored)
 * @param height The request height of the preview (Note: This property may be ignored)
 * @param callback Function that will be called with this signature:
 *		  {"result":status, "path":path} where status is one of "succeeded", "failed" and path is the path to the preview image
 */
adobe.dps.Store.prototype.getPreviewImage = function(folioProductId, portrait, width, height, callback)
{
	var token = this.getResponseToken("getPreviewImage");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getPreviewImage", folioProductId:folioProductId, portrait:portrait, width:width, height:height, token:token}});
};

/**
 * Get receipts for all purchased folios and purchased subscriptions.
 *
 * @param:	callback Function that will be receive a dictionary of Store Product Ids -> receipt pairs in the form of
 *			{productId:receipt}. The  folio product Ids can be retreived by the getFolioData() call.  The receipts
 *			should be in the form the respective store server expects to receive. For subscription receipts, we only
 *			pass the receipt associated with the subscription that was most recently purchased.
 */
adobe.dps.Store.prototype.getReceiptData = function(callback)
{
	var token = this.getResponseToken("getReceiptData");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getReceiptData", token:token}});
};


/**
 *	Returns the orientation registered by the Viewer since the orientation provided
 *  by window.location does not work on all Android devices.
 * 
 *  @param:	callback Function that will receive a JSON with the proper orientation.
 */
 
adobe.dps.Store.prototype.getWindowOrientation = function(callback)
{
	var token = this.getResponseToken("getWindowOrientation");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getWindowOrientation", token:token}});
};

/**
 * Get list of product Ids that are entitled via third-party entitlements.
 *
 * @param:	callback Function that will be receive an array of Store Product Ids.
 */
adobe.dps.Store.prototype.getEntitledProducts = function(callback)
{
	var token = this.getResponseToken("getEntitledProducts");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"getEntitledProducts", token:token}});
};

/**
 * Display the custom subscription popup dialog
 */
adobe.dps.Store.prototype.presentCustomDialog = function()
{
    ADOBEDMPNS.Viewer.bridge.send({event:{type:"displayCustomDialog"}});
};

/**
 * Dismiss the custom subscription popup dialog
 */
adobe.dps.Store.prototype.dismissCustomDialog = function()
{
    ADOBEDMPNS.Viewer.bridge.send({event:{type:"dismissCustomDialog"}});
};

/**
 * Display the welcome screen
 */
adobe.dps.Store.prototype.displayWelcomeScreen = function()
{
    ADOBEDMPNS.Viewer.bridge.send({event:{type:"displayWelcomeScreen"}});
};

/**
 * Dismiss the welcome screen
 */
adobe.dps.Store.prototype.dismissWelcomeScreen = function()
{
    ADOBEDMPNS.Viewer.bridge.send({event:{type:"dismissWelcomeScreen"}});
};

/**
 * Restore purchases.
 */
adobe.dps.Store.prototype.restorePurchases = function(callback)
{    
	var token = this.getResponseToken("restorePurchases");
	this.callbackDictionary[token] = callback;
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"restorePurchases", token:token}});
};

// Internal - helper function used to get a token for the callback
adobe.dps.Store.prototype.getResponseToken = function(funcName)
{
	var date = new Date();
	return funcName + date.getTime();
};

// Internal - function called to process and return data from the viewer
// error is an object which represents any error that could have occurred.
// data will most likely be null if error is not.
adobe.dps.Store.prototype.processResponse = function(data, token)
{
	this.callbackDictionary[token](data);
	delete this.callbackDictionary[token];
};

// Create the actual store instance
adobe.dps.store = new adobe.dps.Store();

// Set the state variable denoting that the context is ready and call
// the user's event handler
window.adobedpscontextloaded = true;

if( window.onadobedpscontextloaded !== undefined )
{
	window.onadobedpscontextloaded();
}
