(function(context){

//fallback to global namespace
var	scope = context || window;

/**
	DOMParser provides methods for translating the DOM into JSON and String.
	JSON representations created with this object can also be reconstructed into an HTML structure.
	
	TODO: implement class-level 'excludedTags' shared across members
**/
var DOMParser = function(){
	this.excludedTags = [];
}

DOMParser.prototype = {
	
	/**
		Convert a DOM representation to a string. Follows nested elements.
		
		@param {HTMLElement} rootNode parent element from which to start the conversion. rootNode is not included in the result;
			Default: document.body
		@param excludedTags {Array} an array of HTML tag names to be excluded from the resulting string. Tags and their contents will be excluded; 
			Example: excludedTags = ["script", "object", "style"]
		
		return {String} the innerHTML property of the rootNode
	**/
	convertDOMToString: function(rootNode, excludedTags){
		var src = this.isValidElement(rootNode)? rootNode : document.body,
			string = src.innerHTML;
		
		//remove specific tags from innerHTML string
		if (excludedTags){
			
			//make sure everything's lowercase
			excludedTags = this.lowerCaseArrayItems(excludedTags);
			
			var len = excludedTags.length;
		
			if (len){
				while (len--){
					var patt = new RegExp('<'+excludedTags[len]+'\\b[^>]*>[\\w\\W]*?<\/'+excludedTags[len]+'>', 'gi');
					string = string.replace(patt, "");
				}
			}
		}
		
		return string;
	},
	
	/**
		Convert a DOM structure to a JSON representation
		
		@param {HTMLElement} rootNode parent element from which to start the node conversion. rootNode is not included in the result;
			Default: document.body
		@param excludedTags {Array} an array of HTML tag names to be excluded from the resulting string. Tags and their contents will be excluded; 
			Example: excludedTags = ["script", "object", "style"]
			
		@return {String} JSON string representing the DOM
	**/
	convertDOMToJSON: function(rootNode, excludedTags){
		var startNode = this.isValidElement(rootNode)? rootNode : document.body;

		if (excludedTags){
			this.excludedTags = this.lowerCaseArrayItems(excludedTags);
		}
		
		return this.parseNode(startNode);
	},
	
	/**
		Check if a node is a valid HTML element

		@param {Object} node object to check
		@return {Boolean} true if the node represents a HTML element
	**/
	isValidElement: function(node){
		return node instanceof HTMLElement;
	},
	
	/**
		Convert all strings from an array to lowercase
		
		@param {Array} array with string items
		@return {Array} the original array with lowercased strings
	**/
	lowerCaseArrayItems: function(array){
		if (array && array.length){
			var len = array.length,
			 	item;
			
			while(len--){
				item = array[len];
				//avoid lowercasing non-strings
				if (typeof item == "string")
					array[len] = item.toLowerCase();	
			}
		}
		
		return array
	},
	
	parseNode: function(node){
		var nodeData = {};
		nodeData.nodeType = node.nodeType;
		
		//extract the tagname
		if (node.tagName){
			nodeData.tagName = node.tagName.toLowerCase();
			
			//avoid parsing this node
			if (this.excludedTags && this.excludedTags.indexOf(nodeData.tagName) != -1){
				return false;	
			}
		}
		
		//text node; get its text value
		if (node.nodeType === 3){
			nodeData.nodeValue = node.nodeValue;
		}
		
		//node has inline attributes (id, style, title, etc..)
		if (node.attributes){
			
			//extract key/value pairs
			nodeData.attributes = this.parseAttributes(node.attributes);
		}
		
		
		//node has child nodes; parse them as well					
		if (node.hasChildNodes()){
			var childNodes = node.childNodes,
				len = childNodes.length,
				i = 0;
				
			nodeData.childNodes = [];
			
			for(; i< len; i++){
				//parseNode has early return that returns undefined;
				//avoid adding dead objects
				var child = this.parseNode(childNodes[i]);
				if (child){
					nodeData.childNodes.push(child)
				}
			}
		}
		
		return nodeData;
	},
	
	/**
		Extract only key/value pairs from an attributes node associated to an HTMLElement
		
		@param {Object/NamedNodeMap} attributes an object that contains HTMLElement attributes
		@returns {Object} an object with key/value pairs corresponding to element attributes.
	**/
	parseAttributes: function(attributes){
		var map = {},
			len = attributes.length;

		if (len){
			while (len--){
				map[attributes[len].name] = attributes[len].value;
			}
		}

		return map;
	},
	
	/**
		Convert a JSON representation to a DOM structure.
		Contains nested nodes and node attributes.
		
		return {DOMFragment}
		
		TODO: implement DOM construction
	**/
	convertJSONToDOM: function(JSONString, callback){
		
	}
}

//get custom namespace
scope.ADOBEDMPNS = scope.ADOBEDMPNS ? scope.ADOBEDMPNS : {};

//expose DOMParser to the scope, under the custom namespace
scope.ADOBEDMPNS.DOMParser = new DOMParser();

})(window)