// Page Template Library
// geometric calculations

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-geom");

var intersectRects = function( r1, r2 )
{
    var x1min = r1.x;
    var x1max = x1min + r1.width;
    var y1min = r1.y;
    var y1max = y1min + r1.height;
    var x2min = r2.x;
    var x2max = x2min + r2.width;
    var y2min = r2.y;
    var y2max = y2min + r2.height;
    var xmin = Math.max(x1min, x2min);
    var xmax = Math.min(x1max, x2max);
    var ymin = Math.max(y1min, y2min);
    var ymax = Math.min(y1max, y2max);
    if( xmin < xmax && ymin < ymax )
        return {x:xmin, y:ymin, width:xmax-xmin, height:ymax-ymin};
    return null;
};

var breakUpRects = function(pgt, arr)
{
    // break up rectangles vertically, so that no rectangle
    // starts "in the middle" of another rectangle in y direction
    // ASSUMPTION: rectangles are sorted in increasing y and then increasing height order
    var i = 1;
    var prev = null;
    while(i < arr.length)
    {
        prev = arr[i-1];
        var curr = arr[i];
        var ins;
        if( prev === null )
        {
            i++;
            continue;
        }
        if( prev.y > curr.y )
            pgt.error("unexpected state");
        if( prev.y == curr.y )
        {
            if( prev.height > curr.height )
                pgt.error("unexpected state");
            if( prev.height == curr.height )
            {
                i++;
                continue;
            }
            // break up curr, move forward
            ins = {x:curr.x, y:curr.y + prev.height, width:curr.width, height:curr.height-prev.height};
            curr.height = prev.height;
            i++;
        }
        else if( prev.y + prev.height > curr.y )
        {
            // break up prev, move backward if possible
            ins = {x:prev.x, y:curr.y, width:prev.width, height:prev.y+prev.height-curr.y};
            prev.height = curr.y - prev.y;
            if( i > 1 )
                i--;
        }
        else
        {
            i++;
            continue;
        }

        // insert a newly created rect
        for( var j = i ; j < arr.length && rectCompareYH(arr[j],ins) < 0; j++ )
        {
            // empty
        }
        arr.splice(j, 0, ins);
    }
};

var mergeRects = function(arr, minDistance)
{
    // horizontally merge rectangles that occupy the same vertical space
    // if they are no more than minDistance apart
    // ASSUMPTION: rectangles don't partially overlap in y direction
    // ASSUMPTION: rectangles are sorted in increasing y and then increasing x order    
    var prev = null;
    for( var i = 0 ; i < arr.length ; i++ )
    {
        var rect = arr[i];
        if( prev !== null && prev.y == rect.y && prev.x + prev.width + minDistance >= rect.x )
        {
            // merge
            prev.width = Math.max(prev.width, rect.x + rect.width - prev.x);
            arr.splice(i,1);
            i--;
        }
        else
        {
            prev = rect;
        }
    }
};

var rectCompareYH = function(r1,r2)
{
    var d = r1.y - r2.y;
    if( d === 0 )
        d = r1.height - r2.height;
    return d;
};

var rectCompareYX = function(r1,r2)
{
    var d = r1.y - r2.y;
    if( d === 0 )
        d = r1.x - r2.x;
    return d;
};

// sort and break down rects so they go in increasing y order and don't intersect
var sortRects = function(pgt, arr, minWidth)
{
    arr.sort(rectCompareYH);
    breakUpRects(pgt, arr);
    arr.sort(rectCompareYX);
    mergeRects(arr, minWidth);    
};

pgt.makeColumnFragments = function( x, y, width, height, overlaps, syntheticFloats, overflowCtrl )
{
    var skip = overflowCtrl=="skip"; // if items that do not fit should skip to the next column/region/page
    
    if( !overlaps || overlaps.length === 0 )
        return [ {pos:{x:x, y:y, width:width, height:height}, skip:skip} ];
   
    var pos = {x:x, y:y, width:width, height:height};
    var floats = [];
    var minWidth = width/3;
    
    for( var i = 0 ; i < overlaps.length ; i++ )
    {
        var orect = overlaps[i];
        var xsect = intersectRects(pos, orect);
        if( xsect === null )
            continue;

        var leftGap = xsect.x - x;
        var rightGap = (x + width) - (xsect.x + xsect.width);
        if( leftGap > 0 && rightGap > 0 )
        {
            // in the middle; cannot handle that, need to be left or right
            if( leftGap <= rightGap )
            {
                xsect.x = x;
                xsect.width += leftGap;
                leftGap = 0;
            }
            else
            {
                xsect.width += rightGap;
                rightGap = 0;
            }
        }
        if( xsect.width > width - minWidth || orect.overlap == "clear" )
        {
            // auto-clear when region is too wide or when explicitly requested 
            xsect.x = x;
            xsect.width = width;
            leftGap = 0;
        }
            
        floats.push(xsect);
    }
    sortRects(this, floats, minWidth);
    
    var widthThr = width - minWidth/2; // subtract minWidth/2 to kill round-off errors
    var fragments = [];
    var floatIndex = 0;
    var fragment = {pos:pos, skip:skip};
    fragments.push(fragment);
    while( floatIndex < floats.length && pos.height > 0 )
    {
        var floatRect = floats[floatIndex++];
        var newY = floatRect.y + floatRect.height;
        if( floatRect.width >= widthThr )
        {
            // float across full column width
            if( floatRect.y > pos.y )
            {
                // finish current fragment
                pos.height = floatRect.y - pos.y;
                // start a new fragment
                pos = {x:x, y:newY, width:width, height:height - (newY-y)};
                fragment = {pos:pos, skip:skip};
                fragments.push(fragment);
            }
            else if( newY <= pos.y )
            {
                this.error("unexpected state");
            }
            else
            {
                // chop off the top of the current fragment
                pos.height -= newY - pos.y;
                pos.y = newY;
            }
        }
        else if( !syntheticFloats )
        {
            // add a float
            if( fragment.floats === undef )
                fragment.floats = [];
            fragment.floats.push({x:floatRect.x - x, y:floatRect.y - pos.y, width:floatRect.width, height:floatRect.height});
        }
        else
        {
            // don't create floats, instead create smaller fragments
            if( pos.y != floatRect.y )
            {
                // finish current fragment
                pos.height = floatRect.y - pos.y;
                // start a new fragment
                pos = {x:x, y:floatRect.y, width:width, height:height - (pos.y-y)};
                fragment = {pos:pos, skip:skip};
                fragments.push(fragment);
            }
            
            // resolve current fragment height
            pos.height = floatRect.height;
            
            // resolve current fragment x and width
            if( x == floatRect.x )
            {
                // left float
                pos.x = floatRect.x + floatRect.width;
                pos.width -= floatRect.width;
                if( floatIndex >= floats.length )
                    floatRect = null;
                else
                {
                    floatRect = floats[floatIndex];
                    if( pos.y == floatRect.y )
                        floatIndex++; // consume
                    else
                        floatRect = null;
                 }
            }
            if( floatRect )
            {
                // right float
                pos.width -= floatRect.width;
            }
            
            // start a new fragment
            pos = {x:x, y:newY, width:width, height:height - (newY-y)};
            fragment = {pos:pos, skip:skip};
            fragments.push(fragment);
        }
    }
    if( pos.height <= 0 )
        fragments.pop();
        
    if( overflowCtrl == "auto" && fragments.length > 0 )
    {
        var tallestFragment = fragments[0];
        for( var fi = 1 ; fi < fragments.length ; fi++ )
        {
            var f = fragments[fi];
            if( f.pos.height > tallestFragment.pos.height )
                tallestFragment = f;
        }
        tallestFragment.skip = false;
    }
        
    return fragments;
};

// create a shape from a string; currently only a list of x,y positions is supported
// for now, don't try to handle things efficiently
var Shape = function( pgt, str )
{
    this.pgt = pgt;
    if( !str )
        return;
    var points = str.split(/[\s,]+/);
    if( points.length % 2 != 0 )
        points.pop();
    this.points = [];
    if( points.length == 0 )
    {
        pgt.error("bad shape: " + str);
        return;
    }
    for( var i = 0 ; i < points.length ; i += 2 )
    {
        var point = {x:points[i]-0, y:points[i+1]-0};
        this.points.push(point);
    }
};

pgt.createShape = function(str)
{
    return new Shape(this, str);
};

var Shape_proto = Shape.prototype;

// scale the shape to a given rect and return a new Shape: x and y values
// 0 to 100 correspond to values x to x+width and y to y+heights respectively 
Shape_proto.scaleToRect = function( rect )
{
    var pts = [];
    var rx = rect.x;
    var rw = rect.width;
    var ry = rect.y;
    var rh = rect.height;
    var points = this.points;
    var count = points.length;
    for( var i = 0 ; i < count ; i++ )
    {
        var p = points[i];
        var q = {x: rx+rw*p.x/100, y: ry+rh*p.y/100};
        pts.push(q); 
    }
    var shape = new Shape();
    shape.points = pts;
    return shape;
};

var sideY = function( y, yTop, yBottom )
{
    if( y < yTop )
        return -1; // above
    if( y > yBottom )
        return 1; // below
    return 0; // inside
};

var intersectY = function( y, y0, x0, y1, x1 )
{
    return x0 + (y-y0)*(x1-x0)/(y1-y0);
};

// convert to rects:
// 2. break the shape into a sequence of rects which vertical coordinates
//    align with y + N*step grid
Shape_proto.toRects = function( y0, step )
{
    var rects = [];
    var points = this.points;
    var pointCount = points.length;
    if( pointCount == 0 )
        return rects;
    var i;
    var y;
    var minY = points[0].y;
    var maxY = minY;
    for( var i = 1 ; i < pointCount ; i++ )
    {
        var y = points[i].y;
        if( y < minY )
            minY = y;
        else if( y > maxY )
            maxY = y;
    }
          
    var currY = y0 + step*Math.floor((minY-y0)/step);
    do
    {
        var prevY = currY;
        currY += step;
        
        // sheck for intersections with prevY-currY stripe
        var p = points[pointCount-1];
        var x1 = p.x;
        var y1 = p.y;
        var side1 = sideY(y1, prevY, currY);
        var minX = 1.0e8;
        var maxX = -1.0e8;
        for( var i = 0 ; i < pointCount ; i++ )
        {
            var x0 = x1;
            var y0 = y1;
            var side0 = side1;
            p = points[i];
            x1 = p.x;
            y1 = p.y;
            side1 = sideY(y1, prevY, currY);
            if( side1 == side0 && side1 != 0 )
                continue; // not intersecting
            var xi;
            if( side0 == -1 || side1 == -1 )
            {
                xi = intersectY(prevY, y0, x0, y1, x1 );
                if( minX > xi )
                    minX = xi;
                if( maxX < xi )
                    maxX = xi;
            }
            if( side0 == 1 || side1 == 1 )
            {
                xi = intersectY(currY, y0, x0, y1, x1 );
                if( minX > xi )
                    minX = xi;
                if( maxX < xi )
                    maxX = xi;
            }
            if( side0 == 0 )
            {
                if( minX > x0 )
                    minX = x0;
                if( maxX < x0 )
                    maxX = x0;
            }            
            if( side1 == 0 )
            {
                if( minX > x1 )
                    minX = x1;
                if( maxX < x1 )
                    maxX = x1;
            }            
        }        
        if( maxX > minX )
            rects.push({x:minX, y:prevY, width:maxX-minX, height:step});
    }
    while( currY < maxY );
    return rects;
};

})(pgt);